﻿using System;
using System.Windows.Forms;
using System.IO;
using DmisLib.Session;

namespace DmisMenu
{
    using Settings = DmisMenu.Properties.Settings;

    public partial class Mainform : Form
    {
        // PCDMIS global variables
        protected PcdmisHelper CMMsession;
        private Boolean PcdmisIsConnected;
        private String DefaultPcdmisPath;

        // PCDMIS delegates
        delegate void ShowEventCallback(String label);
        delegate void DimensionEventCallback(DimensionData data);
        delegate void TerminationEventCallback(int status);
        delegate void ConnectionEventCallback(Boolean status);

        private void InitializePcdmis()
        {
            // Establish connection with PCDLRN and register event handlers.
            PcdmisIsConnected = false;
            try
            {
                CMMsession = new PcdmisHelper(Settings.Default.PcdmisVersion, UNIT_OPTION.MM);
            }
            catch (NoRegistryException e)
            {
                String message = e.Message;
                MessageBox.Show("Incorrect Version of PCDMIS or PCDMIS not found.",
                    "PCDMIS not Found", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            CMMsession.DefaultProbeName = "DefaultTip";
            DefaultPcdmisPath = CMMsession.DefaultPcdmisPath;
            CMMsession.connectPCDLRN();

            // Connect event handlers (delegates) in this code with PCDMIS events
            CMMsession.LocalProgramTerminationEvent += new PcdmisHelper.TerminationEventHandler(PCDLRN_TerminationEvent);
            CMMsession.DimensionEvent += new PcdmisHelper.DimensionEventHandler(PCDLRN_DimensionEvent);
            CMMsession.ShowEvent += new PcdmisHelper.ShowEventHandler(PCDLRN_ShowEvent);
            CMMsession.ConnectedEvent += new PcdmisHelper.ConnectionEventHandler(PCDLRN_ConnectedEvent);
        }

        /* ============================ PCDMIS Event Handlers below this line ========================================*/

        /* Note:  The code below is structured to avoid issues with cross-threading between the thread in which PCDMIS
         * is running and the thread running this .net application. There are two methods created for processing each event
         * type passed from the PcdmisHelper.  The first method is assigned and called by the event handler in the
         * PcdmisHelper. The first method when called invokes the second method which performs the actual work for
         * processing the event.
         * It is the Invoke call which prevents cross-threading exceptions.
         */

        /* Termination Event Methods */
        void PCDLRN_TerminationEvent(int terminationCode)
        {
            TerminationEventCallback cb = new TerminationEventCallback(ProcessTerminationEvent_PCDLRN);
            this.Invoke(cb, new object[] { terminationCode });
        }

        /* Dimension Event Methods */
        void PCDLRN_DimensionEvent(DimensionData data)
        {
            DimensionEventCallback cb = new DimensionEventCallback(ProcessDimensionEvent_PCDLRN);
            this.Invoke(cb, new object[] { data });
        }

        /* Show Events Methods */
        void PCDLRN_ShowEvent(string label)
        {
            ShowEventCallback cb = new ShowEventCallback(ProcessShowEvent_PCDLRN);
            this.Invoke(cb, new object[] { label });
        }

        /* Connected Event Methods */
        void PCDLRN_ConnectedEvent(bool result)
        {
            ConnectionEventCallback cb = new ConnectionEventCallback(ProcessConnectedEvent_PCDLRN);
            this.Invoke(cb, new object[] { result });
        }

        /* ============================ PCDMIS Event Handlers above this line =======================================*/
        /* ============================ PCDMIS Event Handlers below this line ========================================*/
        // These are the four methods which are executed whenever the related PC-DMIS event is triggered. You need to
        // add your own code to perform the action/work you want done each time a PC-Dmis event triggers the method.
        // Note!!! - Keep your code processing time short, PC-Dmis is blocked while your code is processing.  You'll
        // know if the code is taking to long to return to PC-Dmis if you get a pop-up.

        private void ProcessTerminationEvent_PCDLRN(int terminationCode)
        {
            if (terminationCode == 2)
            {
                //TODO Handle CMM operator terminated Pcdmis measurement cycle.
                return;
            }

            //TODO Handle normal termination of Pcdmis measurement cycle.
        }

        private void ProcessDimensionEvent_PCDLRN(DimensionData data)
        {
            //TODO Process dimension data from PCDMIS
            // Sample code demonstrating the Dimension Data received from active PC-Dmis program
            txbDimensionType.Text = data.DimensionType;
            txbDimensionID.Text = data.Id;
            txbNominal.Text = data.Nominal.ToString("F3");
            txbMeasured.Text = data.Measured.ToString("F3");
            txbDeviation.Text = data.Deviation.ToString("F3");
            txbOutTol.Text = data.OutTol.ToString("F3");
        }

        private void ProcessShowEvent_PCDLRN(String label)
        {
            //TODO Handle PCDMIS Show event.
        }

        private void ProcessConnectedEvent_PCDLRN(bool IsConnected)
        {
            PcdmisIsConnected = IsConnected;
            btnPcdmisWindow.Enabled = true;
            //TODO Add any additional action to perform after receiving notice that the PCDLRN is connected.
        }

        /* ============================ PCDMIS Event Handlers above this line =======================================*/
        /* ======================= Application methods & functions below this line ==================================*/
        private void disconnectPCDMIS()
        {
            // Close the PCDLRN runtime service when this application is shutdown.
            if (CMMsession != null)
            {
                CMMsession.UserExit = true;
                CMMsession.Quit();
                CMMsession = null;
            }
        }

        private void MeasurePart(String programName)
        {
            if (CMMsession == null)
            {
                MessageBox.Show("Pcdmis is not connected.", "Pcdmis Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!CMMsession.IsSessionActive)
            {
                MessageBox.Show("Connect is available, Pcdmis is not.", "Pcdmis Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CMMsession.executeAsyncProgram(programName);
        }


    }
}
